import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_getitpayment/src/animateText.dart';
import 'package:flutter_getitpayment/src/card/card_input_formaters.dart';
import 'package:flutter_getitpayment/src/card/card_utils.dart';
import 'package:flutter_getitpayment/src/models/payment_trans_model.dart';
import 'package:flutter_getitpayment/src/models/server_response.dart';
import 'package:flutter_getitpayment/src/pay.dart';

class EnterCard extends StatefulWidget {
  double amountToPay;
  String currency;
  String pageTitle;
  EnterCard({required this.amountToPay,
    this.pageTitle = "Enter Card Details",
    required this.currency});

  @override
  _EnterCardState createState() => _EnterCardState();
}

class _EnterCardState extends State<EnterCard> {
  bool _isLoading = false;
  ServerResponse? result;
  TextEditingController numberController = new TextEditingController();
  TextEditingController address = new TextEditingController();
  PaymentTransaction? _paymentCard;
  final _formKey = GlobalKey<FormState>();
  void _getCardTypeFrmNumber() {
    String input = CardUtils.getCleanedNumber(numberController.text);
    CardType cardType = CardUtils.getCardTypeFrmNumber(input);
    setState(() {
      this._paymentCard!.type = cardType;
    });
  }

  void _validateInputs() async {
    final FormState form = _formKey.currentState!;
    if (!form.validate()) {
      final snackBar = SnackBar(
          backgroundColor: Colors.white,
          content: const Text(
            'Please fix the errors in red before submitting.',
            style: TextStyle(color: Colors.black),
          ));
      ScaffoldMessenger.of(context).showSnackBar(snackBar);
    } else {
      form.save();
      setState(() {
        _isLoading = true;
      });
      // Encrypt and send send card details to src the gateway
      log("card is valid");
      await GetItPayments().pay(paymentTransaction: _paymentCard!, context: context).then((value) {
        setState(() {
          result = value;
        });
      });
    }
  }
  Future<bool> onWillPop() async {
    Navigator.pop(context, result);
    return false;
  }
  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: onWillPop,
      child: Scaffold(
        appBar: AppBar(
          backgroundColor: Colors.white,
          elevation: 0.0,
          title: Text(
            widget.pageTitle,
            style: TextStyle(color: Colors.black),
          ),
          leading: IconButton(
              onPressed: () {
                Navigator.pop(context, result);
              },
              icon: const Icon(
                Icons.arrow_back,
                color: Colors.black,
              )),
        ),
        backgroundColor: Colors.white,
        body: result == null
            ? Form(
          key: _formKey,
          child: ListView(
            children: [
              const Padding(padding: EdgeInsets.only(top: 20)),
              Container(
                margin: const EdgeInsets.symmetric(horizontal: 15),
                padding: const EdgeInsets.symmetric(horizontal: 10),
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(5),
                    border: Border.all(color: Colors.grey.shade300)),
                child: TextFormField(
                  textCapitalization: TextCapitalization.words,
                  style: TextStyle(color: Colors.black),
                  onSaved: (String? value) {
                    _paymentCard!.name = value!.trim();
                  },
                  validator: (value) {
                    if (value == null) {
                      return "Enter name on card";
                    } else {
                      if (value == "") {
                        return "Enter name on card";
                      } else {
                        return null;
                      }
                    }
                  },
                  decoration: const InputDecoration(
                      labelText: "NAME ON CARD", border: InputBorder.none),
                ),
              ),
              const Padding(padding: EdgeInsets.only(top: 10)),
              Container(
                margin: const EdgeInsets.symmetric(horizontal: 15),
                padding: const EdgeInsets.symmetric(horizontal: 10),
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(5),
                    border: Border.all(color: Colors.grey.shade300)),
                child: TextFormField(
                  style: TextStyle(color: Colors.black),
                  keyboardType: TextInputType.number,
                  inputFormatters: [
                    FilteringTextInputFormatter.digitsOnly,
                    LengthLimitingTextInputFormatter(19),
                    CardNumberInputFormatter()
                  ],
                  controller: numberController,
                  decoration: InputDecoration(
                    labelText: "CARD NUMBER",
                    border: InputBorder.none,
                    icon: CardUtils.getCardIcon(_paymentCard!.type!),
                  ),
                  onSaved: (String? value) {
                    _paymentCard!.cardNumber =
                        CardUtils.getCleanedNumber(value!);
                  },
                  validator: CardUtils.validateCardNumWithLuhnAlgorithm,
                ),
              ),
              const Padding(padding: EdgeInsets.only(top: 10)),
              Row(
                children: [
                  Expanded(
                    child: Container(
                      margin: const EdgeInsets.symmetric(horizontal: 15),
                      padding: const EdgeInsets.symmetric(horizontal: 10),
                      decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(5),
                          border: Border.all(color: Colors.grey.shade300)),
                      child: TextFormField(
                        style: TextStyle(color: Colors.black),
                        validator: CardUtils.validateDate,
                        keyboardType: TextInputType.number,
                        inputFormatters: [
                          FilteringTextInputFormatter.digitsOnly,
                          LengthLimitingTextInputFormatter(4),
                          CardMonthInputFormatter()
                        ],
                        decoration: const InputDecoration(
                            labelText: "EXPIRES", border: InputBorder.none),
                        onSaved: (value) {
                          List<int> expiryDate =
                          CardUtils.getExpiryDate(value!);
                          _paymentCard!.month =
                          expiryDate[0].toString().length == 1
                              ? "0" + expiryDate[0].toString()
                              : expiryDate[0].toString();
                          _paymentCard!.year = expiryDate[1].toString();
                        },
                      ),
                    ),
                  ),
                  // Padding(padding: EdgeInsets.only(right: 10),),
                  Expanded(
                    child: Container(
                      margin: const EdgeInsets.symmetric(horizontal: 15),
                      padding: const EdgeInsets.symmetric(horizontal: 10),
                      decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(5),
                          border: Border.all(color: Colors.grey.shade300)),
                      child: TextFormField(
                        style: TextStyle(color: Colors.black),
                        inputFormatters: [
                          FilteringTextInputFormatter.digitsOnly,
                          LengthLimitingTextInputFormatter(4),
                        ],
                        validator: CardUtils.validateCVV,
                        keyboardType: TextInputType.number,
                        decoration: const InputDecoration(
                            labelText: "CVV", border: InputBorder.none),
                        onSaved: (value) {
                          _paymentCard!.cvv = int.parse(value!);
                        },
                      ),
                    ),
                  )
                ],
              ),
              // Padding(
              //   padding: const EdgeInsets.only(top: 10.0),
              //   child: Container(
              //     margin: EdgeInsets.symmetric(horizontal: 15),
              //     padding: EdgeInsets.symmetric(horizontal: 10),
              //     decoration: BoxDecoration(
              //         borderRadius: BorderRadius.circular(5),
              //         border: Border.all(color: Colors.grey.shade300)),
              //     child: TextFormField(
              //       onSaved: (String? value) {
              //         _paymentCard.address = value!.trim();
              //       },
              //       autocorrect: false,
              //       validator: (value) {
              //         if (value == null) {
              //           return "Enter your address";
              //         } else {
              //           if (value == "") {
              //             return "Enter your address";
              //           } else {
              //             return null;
              //           }
              //         }
              //       },
              //       controller: address,
              //       decoration: InputDecoration(
              //           labelText: "Your Address",
              //           border: InputBorder.none),
              //     ),
              //   ),
              // ),
              const Padding(padding: EdgeInsets.only(top: 10)),
              // Padding(padding: EdgeInsets.only(top: 10)),
              // CheckboxListTile(
              //     title: new Text("Save card securely for next payment."),
              //     value: saveCard,
              //     onChanged: (value) {
              //       saveCard = value!;
              //       setState(() {});
              //     }),
              const Padding(padding: EdgeInsets.only(top: 10)),
              Text("Transaction encrypted with latest\nsecurity encryption practices",
              textAlign: TextAlign.center, style: TextStyle(color: Colors.black26)),
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 10),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Padding(
                      padding: const EdgeInsets.only(right: 10.0),
                      child: Image.asset(
                        "assets/card/visa.png",
                        width: 50,
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(right: 10.0),
                      child: Image.asset(
                        "assets/card/mastercard.jpeg",
                        width: 50,
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
        )
            : responseValidator(context),
        bottomNavigationBar: SizedBox(
          height: 100,
          child: Center(
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 20),
              child: Row(
                children: [
                  Expanded(
                    child: result == null
                        ? RaisedButton(
                      elevation: 5.0,
                      padding: const EdgeInsets.symmetric(vertical: 15),
                      onPressed: _isLoading
                          ? null
                          : () {
                        _validateInputs();
                      },
                      color: Theme.of(context).colorScheme.secondary,
                      child: _isLoading
                          ? const CircularProgressIndicator()
                          : Text(
                        "PAY ${widget.amountToPay.toStringAsFixed(2)} ${widget.currency}",
                        style: const TextStyle(
                            color: Colors.black,
                            fontSize: 20,
                            fontWeight: FontWeight.bold),
                      ),
                    )
                        : result!.message == "Approved"
                        ? RaisedButton(
                      elevation: 5.0,
                      padding: const EdgeInsets.symmetric(vertical: 15),
                      child: const Text(
                        "Continue",
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.bold),
                      ),
                      onPressed: () {
                        Navigator.pop(context, result);
                      },
                      color: Colors.black,
                    )
                        : RaisedButton(
                      elevation: 5.0,
                      padding: const EdgeInsets.symmetric(vertical: 15),
                      child: const Text(
                        "Try Again",
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.bold),
                      ),
                      onPressed: () {
                        setState(() {
                          result = null;
                          _isLoading = false;
                        });
                      },
                      color: Colors.black,
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget responseValidator(BuildContext context) {
    return Container(
      child: Center(
        child: AnimateText(
          changeDirection: false,
          delay: 200,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.min,
            children: [
              Padding(
                padding: const EdgeInsets.only(bottom: 10.0),
                child: result!.status == "OK" && result!.message == "Approved"
                    ? Image.asset(
                  "assets/card/payment-complete.gif",
                  width: 100,
                )
                    : Icon(Icons.cancel, color: Colors.red, size: 100),
              ),
              result!.status == "OK" && result!.message == "Approved"
                  ? const Text(
                "Thank You!",
                textAlign: TextAlign.center,
                style:
                TextStyle(fontWeight: FontWeight.bold, color: Colors.black, fontSize: 20),
              )
                  : Text(
                result!.message,
                textAlign: TextAlign.center,
                style:
                TextStyle(fontWeight: FontWeight.bold,
                    color: Colors.black, fontSize: 20),
              ),
              result!.error == null
                  ? Container()
                  : new Text(
                result!.error.toString(),
                style: TextStyle(color: Colors.black38),
              ),
              Padding(padding: EdgeInsets.only(bottom: 10)),
            ],
          ),
        ),
      ),
    );
  }

  @override
  void initState() {
    super.initState();
    _paymentCard = PaymentTransaction(
        type: CardType.Others,
        amount: widget.amountToPay,currency: widget.currency);
    numberController.addListener(_getCardTypeFrmNumber);
  }

  @override
  void dispose() {
    numberController.dispose();
    address.dispose();
    super.dispose();
  }
}